package gwacl

import (
    "errors"
    "fmt"
    . "launchpad.net/gocheck"
)

type httpErrorSuite struct{}

var _ = Suite(&httpErrorSuite{})

func (suite *httpErrorSuite) TestNewHTTPErrorParsesAzureError(c *C) {
    description := "upload failed"
    status := 415
    code := "CannotUpload"
    message := "Unknown data format"
    xml := fmt.Sprintf(`<Error>
        <Code>%s</Code>
        <Message>%s</Message>
    </Error>`,
        code, message)

    httpErr := newHTTPError(status, []byte(xml), description)

    azureErr, ok := httpErr.(*AzureError)
    c.Assert(ok, Equals, true)
    c.Check(azureErr.StatusCode(), Equals, status)
    c.Check(azureErr.Code, Equals, code)
    c.Check(azureErr.Message, Equals, message)
    c.Check(httpErr, ErrorMatches, ".*"+description+".*")
    c.Check(httpErr, ErrorMatches, ".*415: Unsupported Media Type.*")
}

func (suite *httpErrorSuite) TestNewHTTPErrorResortsToServerError(c *C) {
    description := "could not talk to server"
    status := 505

    httpErr := newHTTPError(status, []byte{}, description)

    _, ok := httpErr.(*ServerError)
    c.Assert(ok, Equals, true)
    c.Check(httpErr.StatusCode(), Equals, status)
    c.Check(httpErr, ErrorMatches, ".*505: HTTP Version Not Supported.*")
    c.Check(httpErr, ErrorMatches, ".*"+description+".*")
}

func (suite *httpErrorSuite) TestAzureErrorComposesError(c *C) {
    description := "something failed"
    status := 410
    httpErr := AzureError{
        error:      errors.New(description),
        HTTPStatus: HTTPStatus(status),
        Code:       "MissingError",
        Message:    "Your object has disappeared",
    }
    c.Check(httpErr.Error(), Equals, "something failed: MissingError - Your object has disappeared (http code 410: Gone)")
}

func (suite *httpErrorSuite) TestServerErrorComposesError(c *C) {
    description := "something failed"
    status := 501
    httpErr := ServerError{
        error:      errors.New(description),
        HTTPStatus: HTTPStatus(status),
    }
    c.Check(httpErr.Error(), Equals, "something failed (501: Not Implemented)")
}
